/*
 *
 * Copyright (c) 2001
 *	Politecnico di Torino.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that: (1) source code
 * distributions retain the above copyright notice and this paragraph
 * in its entirety, (2) distributions including binary code include
 * the above copyright notice and this paragraph in its entirety in
 * the documentation or other materials provided with the
 * distribution, and (3) all advertising materials mentioning features
 * or use of this software display the following acknowledgment:
 * ``This product includes software developed by the Politecnico di Torino
 * Telecomunication Networking Group.''
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * 
 * Author:	Marco Mellia, Andrea Carpani
 * 		Telecomunication Networks Group
 * 		Politecnico di Torino
 * 		Torino, Italy
 *              http://http://www.tlc-networks.polito.it/index.html
 *		mellia@mail.tlc.polito.it
 *
 * Tstat is deeply based on TCPTRACE. The authors would like to thank
 * Shawn Ostermann for the development of TCPTRACE.
 * The original TCPTRACE copyright can be found in tcptrace.copyright file.
 *
*/

#include <stdio.h>
#include "tstat.h"

/*
 * Manages all the addresses hit count  
*/

long int tot_adx_hash_count, tot_adx_list_count, adx_search_hash_count,
  adx_search_list_count;

int
add_adx (struct in_addr *adx, struct adx *adx_index[])
{

  unsigned pos;
  struct adx *temp_adx, *ptr_adx, *prev_ptr_adx;
  unsigned long seed;

  seed = (adx->s_addr & ADDR_MASK);
  adx_search_hash_count++;
  pos = (seed % MAX_ADX_SLOTS);

  if (adx_index[pos] == NULL)
    {
      tot_adx_hash_count++;
      /* Inserisco il primo */
      temp_adx = (struct adx *) MMmalloc (sizeof (struct adx), "add_adx");
      temp_adx->next = NULL;
      temp_adx->ip.s_addr = seed;
      temp_adx->hits = 1;
      adx_index[pos] = temp_adx;
      return 1;
    }
  else if (adx_index[pos]->ip.s_addr == seed || adx_index[pos]->ip.s_addr == 0L)
    {
      /* That's a free address which is already available... */
      adx_index[pos]->ip.s_addr = seed;
      adx_index[pos]->hits++;
      return 1;
    }

  ptr_adx = adx_index[pos];
  while (ptr_adx != NULL)
    {
      adx_search_list_count++;
      if (ptr_adx->ip.s_addr == seed || ptr_adx->ip.s_addr == 0L)
	{
          ptr_adx->ip.s_addr = seed;
	  ptr_adx->hits++;
	  return 1;
	}
      /* uso "prev" per tenere a mente l'elemento precedente */
      prev_ptr_adx = ptr_adx;
      ptr_adx = ptr_adx->next;
    }
  /* ...o accodarlo */
  tot_adx_list_count++;
  temp_adx = (struct adx *) MMmalloc (sizeof (struct adx), "add_adx");
  temp_adx->next = NULL;
  temp_adx->ip.s_addr = seed;
  temp_adx->hits = 1;
  prev_ptr_adx->next = temp_adx;
  return 1;
}


int
print_adx (struct adx *adx_index[])
{
  int i;
  struct adx *tmp_adx;
  struct stat fbuf;
  FILE *fp;
  char filename[200];

  /* controlla la directory */
  if (stat (curr_data_dir, &fbuf) == -1)
    {
      printf ("Creating output dir %s\n", curr_data_dir);
      mkdir (curr_data_dir, 0775);

    }

  sprintf (filename, "%s/%s", curr_data_dir, "addresses");
  fp = fopen (filename, "w");

  if (fp == NULL)
    {
      printf ("Could not open file %s\n", filename);
      return 0;
    }

  for (i = 0; i < MAX_ADX_SLOTS; i++)
    {
      tmp_adx = adx_index[i];
      if (tmp_adx != NULL)
	{
	  if (tmp_adx->hits != 0)
	    {
//	      fprintf (fp, "%d ", i);
	      while (tmp_adx != NULL)
		{
		  fprintf (fp, "%s \t %d\n", inet_ntoa (tmp_adx->ip),
			   tmp_adx->hits);
		  tmp_adx->hits = 0;
		  tmp_adx->ip.s_addr = 0L;
		  tmp_adx = tmp_adx->next;
		}
	    }
	}

    }
  fprintf (fp, "\n");
  fclose (fp);
  /* exit with a clean code */
  return (1);
}


/*
** Funzione:	int bubblesort(struct adx *ptr_adx)
** Restituisce:	un intero 0 o 1 a seconda del successo.
** Note:		scorre la lista di strutture adx e le ordina per hits decrescenti.
*/

/* int bubblesort(struct adx *ptr_adx) 
{
	struct adx *temp_adx, *t1_adx, *t2_adx, *head_adx;

	head_adx = ptr_adx;
	if (temp_adx == NULL) return 1;
	if (temp_adx->next == NULL) return 1;

	temp_adx = head_adx;
	while (temp_adx->next != NULL) {
		if (temp_adx->hits < temp_adx->next->hits) {
			t1_adx = temp_adx;
			t2_adx = temp_adx->next->next;
			temp_adx = temp_adx->next;
			temp_adx->next =t1_adx;
			temp_adx->next->next =t2_adx;
			
			temp_adx = head_adx;
		}
	}
	return(1);
}
 */

/* 
** Variazioni sul tema freelist.
**
**
*/
